﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Text;
using MSTBytes.Tools.CSVAdapter;

namespace TestApp
{
    class Program
    {
        static void Main(string[] args)
        {

            CustomFormating_Example();
            CustomParsing_Example();

            FromString_Example();
            FromStringRaw_Example();

            ToFile_Example();
            ToString_Example();

            FromFile_Example();
            FromFileRaw_Example();
        }

        private static void ToString_Example()
        {
            DataTable Data = new DataTable();
            Data.Columns.Add("Item", typeof(string));
            Data.Columns.Add("Quantity", typeof(double));
            Data.Columns.Add("Duedate", typeof(DateTime));
            Data.Columns.Add("Price", typeof(double));

            Data.Rows.Add("I001", 23, new DateTime(2021, 12, 12), 35.1);
            Data.Rows.Add("I002", 22, new DateTime(2021, 12, 31), 1.5);

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            Adapter.DateFormat = "yyyyMMdd";
            string Result = Adapter.ToString(Data);
        }


        private static void CustomFormating_Example()
        {
            DataTable Data = new DataTable();
            Data.Columns.Add("Item", typeof(string));
            Data.Columns.Add("Quantity", typeof(double));
            Data.Columns.Add("Duedate", typeof(DateTime));
            Data.Columns.Add("Price", typeof(double));

            Data.Rows.Add("I001", 23, new DateTime(2021, 12, 12), 35.1);
            Data.Rows.Add("I002", 22, new DateTime(2021, 12, 31), 1.5);

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            Adapter.DateFormat = "yyyyMMdd";
            Adapter.OnValueFormating += Adapter_OnValueFormating;
            string Result = Adapter.ToString(Data);
        }

        private static void Adapter_OnValueFormating(object Value, int RowIndex, int ColumnIndex, DataTable Table, ref string FormatedValue)
        {
            //Custom formating for Duedate column
            if (ColumnIndex == 2)
            {
                DateTime dtValue = (DateTime)Value;
                DateTime ActualDate = DateTime.Now.Date;

                TimeSpan Diff = dtValue - ActualDate;
                FormatedValue = Convert.ToString(Convert.ToInt32(Math.Round(Diff.TotalDays, 0)));
            }
        }

        private static void ToFile_Example()
        {
            DataTable Data = new DataTable();
            Data.Columns.Add("Item", typeof(string));
            Data.Columns.Add("Quantity", typeof(double));
            Data.Columns.Add("Duedate", typeof(DateTime));
            Data.Columns.Add("Price", typeof(double));

            Data.Rows.Add("I001", 23, new DateTime(2021, 12, 12), 35.1);
            Data.Rows.Add("I002", 22, new DateTime(2021, 12, 31), 1.5);

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            Adapter.DateFormat = "yyyyMMdd";
            Adapter.ToFile(Data, "test.csv");

        }

        private static void FromStringRaw_Example()
        {
            string CSVData = "Item,Quantity,DueDate,Price" + Environment.NewLine
            + "I001,23,20211212,35.1" + Environment.NewLine
            + "I002,22,20211231,1.5";

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            DataTable Data = Adapter.FromStringRaw(CSVData);
        }

        private static void FromString_Example()
        {

            string CSVData = "Item,Quantity,DueDate,Price" + Environment.NewLine
            + "I001,23,20211212,35.1" + Environment.NewLine
            + "I002,22,20211231,1.5";

            DataTable Data = new DataTable();
            Data.Columns.Add("Item", typeof(string));
            Data.Columns.Add("Quantity", typeof(double));
            Data.Columns.Add("Duedate", typeof(DateTime));
            Data.Columns.Add("Price", typeof(double));

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            Adapter.DateFormat = "yyyyMMdd";
            Adapter.AllowDBNull = false;
            Adapter.FromString(Data, CSVData);
        }

        private static void CustomParsing_Example()
        {
            string CSVData = "Item,Quantity,DueDate,Price" + Environment.NewLine
            + "I001,23,1,35.1" + Environment.NewLine
            + "I002,22,2,1.5";

            DataTable Data = new DataTable();
            Data.Columns.Add("Item", typeof(string));
            Data.Columns.Add("Quantity", typeof(double));
            Data.Columns.Add("Duedate", typeof(DateTime));
            Data.Columns.Add("Price", typeof(double));

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            Adapter.DateFormat = "yyyyMMdd";
            Adapter.AllowDBNull = false;
            Adapter.OnValueParsing += Adapter_OnValueParsing;
            Adapter.FromString(Data, CSVData);
        }

        private static void Adapter_OnValueParsing(string Value, int RowIndex, int ColumnIndex, DataTable Table, ref object ParsedValue)
        {
            //Custom parse for Duedate column
            if (ColumnIndex == 2)
            {
                DateTime ActualDate = DateTime.Now.Date;
                int Days;

                if (String.IsNullOrEmpty(Value))
                {
                    ParsedValue = ActualDate;
                }
                else if (Int32.TryParse(Value, out Days))
                {
                    ParsedValue = ActualDate.AddDays(Days);
                }
                else
                {
                    ParsedValue = DBNull.Value;
                }

            }
        }

        private static void FromFileRaw_Example()
        {
            /* 
             * File content:
             * Item,Quantity,DueDate,Price
             * I001,23,20211212,35.1
             * I002,22,20211231,1.5
            */

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            DataTable Data = Adapter.FromFileRaw("test.csv");
        }

        private static void FromFile_Example()
        {
            /* 
              * File content:
              * Item,Quantity,DueDate,Price
              * I001,23,20211212,35.1
              * I002,22,20211231,1.5
             */

            DataTable Data = new DataTable();
            Data.Columns.Add("Item", typeof(string));
            Data.Columns.Add("Quantity", typeof(double));
            Data.Columns.Add("Duedate", typeof(DateTime));
            Data.Columns.Add("Price", typeof(double));

            CSVTableAdapter Adapter = new CSVTableAdapter(1);
            Adapter.Delimiter = ",";
            Adapter.DateFormat = "yyyyMMdd";
            Adapter.FromFile(Data, "test.csv");
        }
    }
}
